#!/usr/bin/env python

#############################################################################
#
#   Scheduler Process Client
#
#   Execute events at regular intervals. The schedule is defined in
#   the configuration file. Events are listed, one per line, in the
#   'events' parameter. Each line has a start time, duration, and
#   event name, separated by a the '|' character:
#
#   events:     00:00   |   00:45       | call-remote-1
#               05:00   |   00:45       | call-remote-2
#
#   The times are relative (timedelta) from the start of the event
#   sequence. The entire sequence is repeated at a rate defined by the
#   repear.* parameters.
#
#   The individual events are listed as 'event.<name>.<parameter>' The
#   specific parameters are dependent on the event types. Events are
#   objects that inherit from the Event class. An application would
#   provide it's own Event classes, passed in as the EventFactory
#   object to the Schedule init. Event object must support three
#   methods:
#
#       start   - called once at program startup
#       run     - called each time the event is scheduled
#       stop    - called once on exit
#
#   2005-04-07  Todd Valentic
#               Initial implementation.
#
#   2005-04-19  Todd Valentic
#               Added ability to start in mid-event sequence
#
#   2005-06-08  Todd Valentic
#               Added ability to start events early. This is useful
#                   in cases where multiple events are stacked up
#                   and some take less then there alloted time. It
#                   gives the extra time to the next event. Each
#                   event can be individually controlled, so those
#                   that need to happen at exact times still can.
#                   Even though an event may start early, it still
#                   runs until it's original end time.
#
#   2007-07-14  Todd Valentic
#               Computed offset in the wrong direction. Fixed.
#
#   2008-07-25  Todd Valentic
#               Add small offset in waituntil() to prevent problems
#                   due to small rounding errors.
#
#   2008-08-15  Todd Valentic
#               Still having problems with waituntil(). Now using
#                   a more robust scheme that will continue to
#                   retry/sleep until the target time is reached.
#
#   2009-11-24  Todd Valentic
#               Use currentTime()
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#
#############################################################################

import datetime
import sys

from datatransport import ProcessClient
from datatransprot import ConfigComponent
from datatransport.utilities import datefunc


class Event(ConfigComponent):
    def __init__(self, name, parent):
        ConfigComponent.__init__(self, "event", name, parent)
        self.startearly = self.get_boolean("startearly", False)

    def __getrunning(self):
        return self.parent.running

    running = property(__getrunning)

    def start(self):
        pass

    def stop(self):
        pass

    def run(self, endtime):
        pass


class Scheduler(ProcessClient):
    def __init__(self, argv, EventFactory):
        ProcessClient.__init__(self, argv)

        self.rate = self.get_timedelta("repeat.rate", ":60")
        self.offset = self.get_timedelta("repeat.offset")
        self.sync = self.get_boolean("repeat.sync", True)
        self.whole = self.get_boolean("repeat.whole", False)
        self.exit_on_error = self.get_boolean("exitOnError", False)

        self.events = {}
        self.schedule = []

        for entry in self.get("repeat.events").split("\n"):

            try:
                start, duration, name = entry.split("|")

                start = datefunc.parse_timedelta(start)
                duration = datefunc.parse_timedelta(duration)
                name = name.strip()

                if name not in self.events:
                    self.events[name] = EventFactory(name, self)

                self.schedule.append((start, duration, name))

            except:
                self.log.exception("Problem parsing event")
                self.abort()

        self.log.info("Repeat rate  : %s" % self.rate)
        self.log.info("Repeat offset: %s" % self.offset)
        self.log.info("Repeat sync  : %s" % self.sync)
        self.log.info("Repeat whole : %s" % self.whole)

        self.log.info("Schedule:")

        for start, duration, name in self.schedule:
            self.log.info("  %s: %s - %s" % (name, start, start + duration))

    def waituntil(self, starttime):

        # Sometimes small rounding errors creep in, so spin
        # around until we are really at the target time

        while self.current_time() < starttime:
            if not self.wait(starttime - self.current_time()):
                return False

        return True

    def main(self):

        for event in self.events.values():
            event.start()

        error = False

        if self.sync and self.whole:
            self.log.info("Waiting to sync whole event sequence")
            self.wait(self.rate, self.offset, self.sync)
        else:
            self.log.info("Starting in middle of event sequence")

        while not error and self.is_running():

            basetime = self.current_time()

            if self.sync:
                rate = self.rate.total_seconds()
                inset = datetime.timedelta(seconds=basetime.timestamp() % rate)
                basetime = basetime - inset + self.offset

            self.log.info("---- Starting event cycle ----")

            for start, duration, name in self.schedule:

                event = self.events[name]
                starttime = basetime + start
                endtime = starttime + duration

                self.log.info("start time: %s" % starttime)
                self.log.info("end   time: %s" % endtime)

                if not event.startearly:
                    if not self.waituntil(starttime):
                        break

                self.log.info("Processing %s" % name)

                if self.current_time() >= endtime:
                    self.log.info("  past end time, skipping")
                    continue

                try:
                    event.run(endtime)
                except:
                    self.log.exception("Problem running event")
                    if self.exit_on_error:
                        error = True
                        break

                self.log.info("  finished action for %s" % name)

            self.log.info("=" * 35)

            self.wait(self.rate, self.offset, self.sync)
            self.wait(1)

        for event in self.events.values():
            event.stop()

        if error:
            self.abort()


def main():
    Scheduler(sys.argv).run()



#!/usr/bin/env python3

##########################################################################
#
#   Posting example
#
#   Periodically post a message to a news group 
#
#   2022-10-10  Todd Valentic
#               Initial implementation
#
##########################################################################

from datatransport import ProcessClient
from datatransport import NewsPoster 

import sys

class Client(ProcessClient):

    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poster = NewsPoster(self)

    def init(self):

        self.rate = self.get_rate('rate')
        self.text = self.get('text')

        self.log.info(f'Post message every {self.rate.period}')

    def main(self):
        super().init()

        while self.wait(self.rate):
            self.news_poster.post_text(self.text)
            self.log.info('Posted message')

if __name__ == '__main__':
    Client(sys.argv).run()




#####################################################################
#
#   WatchURL
#
#   This script watches a URL (which, for example, can be a web
#   page or an image) and posts it to the specified newsgroup when
#   a new version appears.
#
#   Based on code written by Angela Li (SRI International).
#
#   History:
#
#   1.0.0   2002-01-15  TAV
#           Initial implementation.
#
#   1.0.1   2002-09-03  Todd Valentic
#           Moved to python2, sri.transport -> Transport
#           Use python2 md5's hexdigest() (replace local hexstr()).
#           Added retry time for problem sites
#           Added header checks, exclude list
#           Added indirect url features.
#
#   1.0.2   2002-10-13  Todd Valentic
#           Added ability to create thumbnails. These are posted
#               as another attachment in the message. Makes it
#               nice when used with ArchiveGroups and the history
#               function.
#
#   1.0.3   2003-03-02  Todd Valentic
#           Added ability to sync the pollrate.
#
#   1.0.4   2003-03-08  Todd Valentic
#           Put the news post in try..except block to catch errors.
#
#   1.0.5   2003-03-31  Todd Valentic
#           Added offset to compliment sync.
#
#   1.0.6   2003-04-16  Todd Valentic
#           Added use of timeoutsocket
#
#   1.0.7   2003-08-25  Todd Valentic
#           Use DeltaDateTime objects in config
#           Added ability to exclude image names in patterns check
#           Added images in anchor tags to those checked
#           Added small test for parser at end of file
#
#   1.0.8   2005-08-15  Todd Valentic
#           Change sync to be a boolean.
#
#   1.0.8   2009-05-11  Todd Valentic
#           md5 module deprecated in Python 2.6. Use a import check
#               to pull in right library on older systems.
#
#   1.0.8   2013-01-29  Todd Valentic
#           Add timeout parameter
#
#   1.0.9   2013-08-18  Todd Valentic
#           Add time replacement to rename rules
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#               Python3 migration:
#                   commands -> subprocess
#                   <> -> !=
#                   htmllib -> html.parser
#                   urlparse -> urllib.parse
#                   removeFile -> remove_file
#                   NewsPoster
#
#####################################################################

import datetime
import fnmatch
import os
import socket
import subprocess
import sys
import time
import urllib

from html.parser import HTMLParser
from hashlib import md5
from urllib.parse import urlparse, urljoin

from datatransport import ProcessClient
from datatransport import NewsPoster
from datatransport.utilities import remove_file


class Parser(HTMLParser):
    def __init__(self, include_names, exclude_names):
        HTMLParser.__init__(self)

        self.include_patters = include_names
        self.exclude_patterns = exclude_names

    def get_images(self, url):

        self.urls = []
        self.base = url

        page = urllib.urlopen(url).read()
        self.reset()
        self.feed(page)
        self.close()

        return self.urls

    def match(self, url):

        try:
            path = urlparse(url)[2]
        except:
            return 0

        for pattern in self.exclude_patterns:
            if fnmatch.fnmatch(path, pattern):
                return 0

        for pattern in self.include_patters:
            if fnmatch.fnmatch(path, pattern):
                return 1

    def handle_image(self, src, alt, ismap, align, width, height):

        # This is a fix for one broken site

        src = src.replace("\n", "")
        src = src.replace("\r", "")

        if self.match(src):
            self.urls.append(urljoin(self.base, src))

        if self.anchor and self.match(self.anchor, self.patterns):
            self.urls.append(urljoin(self.base, self.anchor))

    def anchor_bgn(self, href, name, type):

        if self.match(href):
            self.urls.append(urljoin(self.base, href))


class WatchURL(ProcessClient):
    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poster = NewsPoster(self)

        self.pollrate = self.get_timedelta("pollrate", 600).seconds
        self.sync = self.get_boolean("pollrate.sync", False)
        self.offset = self.get_timedelta("pollrate.offset", 0).seconds
        self.retryrate = self.get_timedelta("retryrate", 60).seconds
        self.timeout = self.get_timedelta("timeout", 60).seconds

        self.url = self.get("url", None)
        self.include_names = self.get("images", "*").split()
        self.exclude_names = self.get("images.exclude", "").split()
        self.check_headers = self.get_boolean("headers.enable", True)
        self.exclude_headers = self.get("headers.exclude", "Date").split()
        self.rename_rules = self.get("rename", "[]")
        self.save_body = self.get_boolean("save.body", True)
        self.save_images = self.get_boolean("save.images", False)
        self.save_thumbnails = self.get_boolean("save.thumbnails", False)
        self.thumbnail_cmd = self.get(
            "thumbnails.cmd", "convert -geometry 125x125 -sharpen 2x2"
        )
        self.thumbnail_exts = self.get("thumbnails.ext", ".jpg .png .gif").split()
        self.thumbnail_name = self.get("thumbnails.name", "%s-thumbnail.jpg")

        socket.setdefaulttimeout(self.timeout)

        try:
            self.rename_rules = eval(self.rename_rules)
        except:
            self.log.exception("Problem parsing the rename rules:")
            self.abort("Aborting")

        if self.url is None:
            self.abort("No URL specified")

        scheme = urlparse(self.url)[0]

        if self.save_images and scheme != "http":
            self.abort("Can only save images from http sites")

        self.parser = Parser(self.include_names, self.exclude_names)

    def gather_urls(self):

        urls = []

        if self.save_body:
            urls.append(self.url)

        if self.save_images:
            urls.extend(self.parser.get_images(self.url))

        self.log.debug("Checking these URLs:")
        for url in urls:
            self.log.debug("  %s" % url)

        return urls

    def get_headers(self, url):

        self.log.debug("  getting headers from %s" % url)

        headers = urllib.urlopen(url).info().headers

        keepers = []

        for header in headers:

            key = header.split(":")[0].strip()

            if key not in self.exclude_headers:
                self.log.debug("    including %s" % header.strip())
                keepers.append(header)
            else:
                self.log.debug("    excluding %s" % header.strip())

        return "".join(keepers)

    def headers_changed(self, urls):

        self.log.debug("Checking headers")

        try:
            prevChecksum = open("checksum.headers").read()
        except:
            prevChecksum = 0

        checksum = md5()

        for url in urls:
            checksum.update(self.get_headers(url))

        checksum = checksum.hexdigest()

        self.log.debug("  prev checksum: %s" % prevChecksum)
        self.log.debug("  new  checksum: %s" % checksum)

        if checksum != prevChecksum:
            self.log.debug("  header change detected")
            open("checksum.headers", "w").write(checksum)
            return 1
        else:
            return 0

    def files_changed(self, checksum):

        self.log.debug("Checking file checksum")

        try:
            prevChecksum = open("checksum.files").read()
        except:
            prevChecksum = 0

        self.log.debug("  prev checksum: %s" % prevChecksum)
        self.log.debug("  new  checksum: %s" % checksum)

        if checksum != prevChecksum:
            self.log.debug("  new files detected")
            open("checksum.files", "w").write(checksum)
            return 1
        else:
            self.log.debug("  files are the same")
            return 0

    def remap(self, filename):

        for pattern, result in self.rename_rules:
            if fnmatch.fnmatch(filename, pattern):
                return self.current_time().strftime(result)

        return filename

    def retrieve_files(self, urls):

        filenames = []
        unknownext = 0
        checksum = md5()

        self.log.debug("Retrieving files:")

        for url in urls:

            path = urlparse.urlparse(url)[2]
            filename = os.path.basename(path)
            if not filename:
                filename = "unknown.%d" % unknownext
                unknownext = unknownext + 1

            filename = self.remap(filename)

            try:
                contents = urllib.urlopen(url).read()
                checksum.update(contents)
                open(filename, "w").write(contents)
                filenames.append(filename)
                self.log.debug("  %s -> %s" % (url, filename))
            except:
                self.log.error("Problem retrieving %s" % url)
                self.log.exception("Traceback:")

        return filenames, checksum.hexdigest()

    def make_thumbnails(self, filenames):

        self.log.debug("Making thumbnails")

        thumbnails = []

        for filename in filenames:
            root, ext = os.path.splitext(filename)
            if ext in self.thumbnail_exts:
                self.log.debug("  creating thumbnail for %s" % filename)
                outputname = self.thumbnail_name % root
                cmd = "%s %s %s" % (self.thumbnail_cmd, filename, outputname)
                status, output = subprocess.getstatusoutput(cmd)

                if status == 0:
                    thumbnails.append(outputname)
                else:
                    self.log.error("Problem running thumbnail command: %s" % cmd)
                    self.log.error("Output: %s" % output)

        return thumbnails

    def main(self):

        if self.sync:
            self.wait(self.pollrate, sync=True, offset=self.offset)

        while self.is_running():

            try:
                urls = self.gather_urls()
            except:
                self.log.error("Problem gathering URLs")
                self.log.exception("Traceback:")
                self.wait(self.retryrate)
                continue

            if not urls:
                self.log.debug("No urls to check")
                self.wait(self.pollrate, sync=self.sync, offset=self.offset)
                continue

            try:
                if self.check_headers and not self.headers_changed(urls):
                    self.log.debug("The headers have not changed")
                    self.wait(self.pollrate, sync=self.sync, offset=self.offset)
                    continue
            except:
                self.log.error("Problem checking headers")
                self.log.exception("Traceback:")
                self.wait(self.retryrate)
                continue

            filenames, checksum = self.retrieve_files(urls)

            if self.files_changed(checksum):
                if self.save_thumbnails:
                    try:
                        filenames.extend(self.make_thumbnails(filenames))
                    except:
                        self.log.exception("Problem making thumbnails")

                if filenames:
                    try:
                        self.news_poster.post(filenames)
                        self.log.info("New files detected, posting %s" % filenames)
                    except:
                        self.log.exception("Error posting to the news server")
                else:
                    self.log.info(
                        "Strange, filenames detected, but none listed for posting..."
                    )

            remove_file(filenames)

            self.wait(self.pollrate, sync=self.sync, offset=self.offset)


def main():
    WatchURL(sys.argv).run()



#!/usr/bin/env python

##########################################################################
#
#   Directory Service
#
#   The directory is used to return information about the
#   various services. It is used primarily in bootstraping
#   by client programs. Based on AMISR directory service.
#
#   2005-11-08  Todd Valentic
#               Initial implementation.
#
#   2006-06-14  Todd Valentic
#               Add label to XMLRPCServerMixin call
#
#   2006-09-16  Todd Valentic
#               Added hold to Service.
#
#   2007-08-19  Todd Valentic
#               Allow directory port to be set.
#
#   2013-01-30  Todd Valentic
#               Make sure port has a value.
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#
##########################################################################

import os
import sys

from datatransport import ProcessClient
from datatransport import ConfigComponent
from datatransport import XMLRPCServer


class Service(ConfigComponent):
    def __init__(self, name, config, parent, **kw):
        ConfigComponent.__init__(self, "service", name, config, parent, **kw)

        self.host = self.get("host", "localhost")
        self.port = self.get_int("port")
        self.label = self.get("label")
        self.url = "http://%s:%s" % (self.host, self.port)
        self.hold = self.get_boolean("hold", True)

        if not self.port:
            raise ValueError("No port specified")


class DirectoryService(ProcessClient):
    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        port = self.get_int("directory.port", 8411)

        self.xmlserver = XMLRPCServer(self, 
            queue_size=100, port=port, label="Directory"
        )
        self.main = self.xmlserver.main

        self.xmlserver.register_function(self.list)
        self.xmlserver.register_function(self.lookup, "get")

        self.services = self.get_components("services", factory=Service)

        self.log.info("Services:")
        for service in self.services.values():
            self.log.info(f"  - {service.label} ({service.host}:{service.port})")

    def list(self):
        return list(self.services)

    def lookup(self, name, key):

        if name in self.services:
            return getattr(self.services[name], key)

        raise KeyError("Unknown service: %s" % name)


def main():
    DirectoryService(sys.argv).run()



#!/usr/bin/env python3

##########################################################################
#
#   Hello World example
#
#   Periodically print a message to the log file.
#
#   2022-10-07  Todd Valentic
#               Initial implementation
#
##########################################################################

from datatransport import ProcessClient

import sys

class Client (ProcessClient):

    def __init__(self, args):
        ProcessClient.__init__(self, args)

    def init(self):

        self.rate = self.get_rate('rate')
        self.text = self.get('text')

        self.log.info(f'Log message every {self.rate.period}')

    def main(self):

        while self.wait(self.rate):
            self.log.info(self.text)

if __name__ == '__main__':
    Client(sys.argv).run()




#!/usr/bin/env python
"""Test confif parser get_bytes"""

##########################################################################
#
#   Test get_bytes config settings
#
#   2022-11-02  Todd Valentic
#               Initial implementation
#
##########################################################################

import sys

from datatransport import ProcessClient


class Demo(ProcessClient):
    """Test program"""

    def __init__(self, args):
        ProcessClient.__init__(self, args)

    def init(self):
        super().init()

        for key in self.options():
            if key.startswith('test.'):
                try:
                    value = self.get_bytes(key)
                    self.log.info("%s: %s", key, self.get_bytes(key))
                except:
                    self.log.error('Problem parsing %s' % key)

if __name__ == "__main__":
    Demo(sys.argv).run()



############################################################################
#
#   PostDataFiles
#
#   This script watches for new radar data files to appear and posts them
#   to the newsserver. There is an option not to remove the original file
#   under the assumption that someone else will in the future. It can also
#   compress the files prior to posting using bzip to reduce their size
#   (we get about a 30% reduction in size on binary MEDAC data).
#
#   The data files are assumed to have a name with the following format:
#
#       <index>.<ext>
#
#   The <index> is a monotonically increasing value that can be tracked to
#   determine if the file has been posted before. The <ext> can be any
#   arbitrary string. The algorithm for determing which new files to post
#   is based on the finding files in the specified directory and with the
#   specified name that are newer since the last time the poll was taken.
#   Of these new files, the index values are compared to the previously
#   posted file and only newer files are posted.
#
#   It is assumed that the newest file in the directory is still being
#   written to by the data aqcuisition system and shouldn't be posted.
#   You can override this behaviour with the "include_current" option.
#
#   1.0.0   2001-02-20  Todd Valentic
#           Initial implementation.
#
#   1.0.1   2001-11-30  Todd Valentic
#           Quote the the filename pattern in the poll() method.
#           Added some debugging message to verbose level 2.
#
#   1.0.2   2001-12-02  Todd Valentic
#           Only update the timestamp file as each source file is
#               processed. Update the time stamp not with the current
#               time, but the time of the source file.
#           Added signal handling to make sure we done a clean exit.
#
#   2.0.0   2002-01-18  Todd Valentic
#           Generalized from PostDataFiles to handle multiple types of
#               radar data.
#
#   2.0.1   2002-03-22  Todd Valentic
#           Remove signal handling (use ProcessClient's).
#           Use commands.getstatusoutput() in compress call.
#
#   2.0.2   2002-03-28  Todd Valentic
#           Change default value of startCurrent to false.
#
#   2.0.3   2002-08-15  Todd Valentic
#           Changed the comparison to handle generic string operations.
#               The filename does not need to be strictly a number anymore.
#           Made file compression optional. The conf option is "compress"
#               and defaults to true.
#           Added option to remove original file (removeFile). Default 0.
#           Added option to only post last N files found (default to all).
#
#   2.0.4   2002-08-27  Todd Valentic
#           Setup with configure script.
#           sri.transport -> Transport
#           split runtime into script and class module.
#
#   2.0.5   2003-04-16  Todd Valentic
#           Use string methods instead of module.
#
#   2.0.6   2003-04-18  Todd Valentic
#           Added ability to specify pollrate in days/hours/mins/secs.
#           Added sync option for wait.
#           If the timestamp file doesn't exist, initialize its time
#               to something really old (Jan 1, 1975). We start off
#               by looking for files newer then that file (unless
#               startCurrent==1). It used to be initialized to the
#               current time and old files never got pulled in when
#               first ran.
#
#   2.0.7   2003-05-02  Todd Valenic
#           Fixed bug in log reporting when a file had already been
#               processed (it was trying to print a string as an int).
#
#   2.0.8   2003-05-03  Todd Valentic
#           Added check_index to turn off index checking.
#
#   2.0.9   2003-06-20  Todd Valentic
#           Added filedate callback.
#
#   2.0.10  2003-08-14  Todd Valentic
#           createCallback -> get_callback
#           createDeltaTime -> getDeltaTime
#
#   2.0.11  2004-02-13  Todd Valentic
#           Skip compression if file is already compressed.
#           Add a maximum file size limit.
#
#   2.0.12  2004-05-07  Todd Valentic
#           Print input path/name to log at startup.
#
#   2.0.13  2004-05-28  Todd Valentic
#           Fixed bug when reporting compression ratio and the
#               input file is 0 length.
#
#   2.0.14  2004-12-27  Todd Valentic
#           Convert from mx.DateTime to datetime
#
#   2.0.15  2005-04-03  Todd Valentic
#           Use library functions to replace external calls to touch
#           Use library bz2 module
#
#   2.0.16  2005-05-04  Todd Valentic
#           Fix bug when timestamp file not present.
#           Changed a number of config parameters to booleans
#
#   2006-10-14  Todd Valentic
#               Convert headers in post to a dict from a list
#
#   2009-11-24  Todd Valentic
#               Use current_time()
#
#   2015-11-23  Todd Valentic
#               Add md5 checksum header.
#
#   2015-11-30  Todd Valentic
#               Use get_rate() for pollrate
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#               Use get_ config methods
#               Python3 updates:
#                   commands -> subprocess
#                   removeFile -> remove_file
#                   sizeDesc -> size_desc
#                   currentTime -> current_time
#                   NewsPoster
#
############################################################################

import bz2
import glob
import math
import os
import subprocess
import sys
import time

from datetime import datetime

from datatransport import ProcessClient
from datatransport import NewsPoster
from datatransport.utilities import size_desc


class PostDataFiles(ProcessClient):
    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poster = NewsPoster(self)

        self.input_name = self.get("input.name", "*.DAT")
        self.input_path = self.get("input.path", ".")
        self.include_current = self.get_boolean("include_current", False)
        self.compress = self.get_boolean("compress", 1)
        self.remove_files = self.get_boolean("remove_files", False)
        self.max_files = self.get_int("max_files", None)
        self.max_size = self.get_bytes("max_size", "20Mb")
        self.check_index = self.get_boolean("check_index", True)
        self.check_size = self.get_boolean("check_size", True)

        self.pollrate = self.get_rate("pollrate", "5:00")
        self.filedate = self.get_callback("filedate", self.filedate_callback)

        self.timefile = "timestamp"
        self.indexfile = "index"

        if not os.path.isfile(self.timefile):
            # Default to sometime long ago
            open(self.timefile, "w").write("0")
            t = time.mktime((1975, 1, 1, 0, 0, 0, 0, 0, 0))
            os.utime(self.timefile, (t, t))

        if self.get_boolean("startCurrent", 0) == 1:
            os.utime(self.timefile, None)

        if not os.path.isfile(self.indexfile):
            open(self.indexfile, "w").write("0")

        self.log.info("Input path: %s" % self.input_path)
        self.log.info("Input name: %s" % self.input_name)

    def filedate_callback(self, file):
        return None

    def split(self, filename):

        filesize = os.path.getsize(filename)

        if not self.check_size or filesize <= self.max_size:
            return [filename]

        numchunks = int(math.ceil(filesize / float(self.max_size)))
        chunks = []
        file = open(filename, "rb")

        for chunk in range(numchunks):
            data = file.read(self.max_size)
            chunkname = "chunk.%03d" % chunk
            open(chunkname, "wb").write(data)
            chunks.append(chunkname)

        self.log.info("  - split into %d parts" % numchunks)

        return chunks

    def get_checksumg(self, filename):
        # Shell out - some files might be quite large
        status, output = subprocess.getstatusoutput("md5sum %s" % filename)

        if status != 0:
            return None

        return output.split()[0]

    def post(self, filename, timestamp):

        files = self.split(filename)
        part = 1
        numparts = len(files)
        basename = os.path.basename(filename)
        checksum = self.get_checksumg(filename)

        for file in files:

            headers = {}
            if numparts > 1:
                headers["X-Transport-Part"] = "%d/%d" % (part, numparts)
                headers["X-Transport-Filename"] = basename
                part += 1

            if checksum:
                # Note - checksum of the *entire* file, not part
                headers["X-Transport-md5"] = checksum

            filesize = os.path.getsize(file)
            self.log.debug("  - posting %s (%s)" % (file, size_desc(filesize)))
            self.news_poster.post([file], date=timestamp, headers=headers)

            if numparts > 1:
                os.remove(file)

    def process(self, file):

        # Compress the file

        self.log.info("Processing %s" % file)

        bzipext = ".bz2"
        basename = os.path.basename(file)
        baseext = os.path.splitext(basename)[1]
        isCompressed = baseext == bzipext
        zipname = basename + bzipext
        lastindex = open(self.indexfile).readline()

        if self.check_index:
            try:
                curindex = os.path.splitext(basename)[0]
            except:
                self.log.error("Cannot determine index of %s" % basename)
                return

            if curindex <= lastindex:
                self.log.info(
                    "  - the file %s has been processed before (index=%s)"
                    % (basename, lastindex)
                )
                return

        try:
            date = self.filedate(file)
            if date:
                self.log.debug("  - file date: %s" % date)
        except:
            self.log.exception("Error calling routine to determine file date")
            return

        if self.compress and not isCompressed:

            starttime = self.current_time()

            self.log.debug("  - compressing file")
            data = open(file).read()
            open(zipname, "w").write(bz2.compress(data))

            orgsize = os.path.getsize(file)
            zipsize = os.path.getsize(zipname)

            if orgsize > 0:
                zippct = (zipsize / float(orgsize)) * 100
            else:
                zippct = 0

            totaltime = self.current_time() - starttime
            self.log.info(
                "  - %s -> %s (%d%%) %s"
                % (size_desc(orgsize), size_desc(zipsize), zippct, totaltime)
            )

            postfile = zipname

        else:

            postfile = file

        # Post file to news server

        try:
            self.post(postfile, date)
        except:
            self.log.exception("Problem post file")
            return

        # Cleanup files

        remove_file(zipname)

        if self.remove_files:
            try:
                remove_file(file)
                self.log.debug("  - removed original file: %s" % file)
            except:
                self.log.exception("Problem removing file: %s" % file)

        # Update the index file

        if self.check_index:
            file = open(self.indexfile, "w").write(str(curindex))

    def poll(self):

        cmd = 'find %s -newer %s -type f -name "%s" -print' % (
            self.input_path,
            self.timefile,
            self.input_name,
        )

        (status, filelist) = subprocess.getstatusoutput(cmd)  # run command, get output

        if status != 0:
            return []

        filelist = filelist.split()  # make list, break at \n
        filelist.sort()

        if not self.include_current:
            filelist = filelist[0:-1]  # don't include the current file

        if self.max_files:  # keep only the last N files
            filelist = filelist[-self.max_files :]

        return filelist

    def do_run(self):

        try:
            files = self.poll()
            self.log.debug("Polling - found %d new files." % len(files))

            for file in files:
                timestamp = os.path.getmtime(file)
                self.process(file)
                os.utime(self.timefile, (timestamp, timestamp))
                if self.is_stopped():
                    return

        except:
            self.log.exception("Failure in run")

    def main(self):

        while self.wait(self.pollrate):
            self.do_run()

        self.log.error("Exiting")


def main():
    PostDataFiles(sys.argv).run()



#!/usr/bin/env python

##############################################################################
#
# FileStore
#
# This script collects data from the polled newsgroup and transfers the
# files into directories on the destination server. The output files are
# written into directories with the following format:
#
# path/name
#
# In the default case, these translate into:
#
# ./YYYY-DDD-hh-mm-ss-filename
#
#  YYYY = Year
#  DDD  = Day of year (001-366)
#  hh   = Hour (00-23)
#  mm   = Minute (00-59)
#  ss   = Second (00-59)
#
# There is a directory structure formed from the date. The filename has
# date and time prefixed to the filename in the message. This last part
# is done to ensure that each file has a unique name even if the file
# that was originally posted always has the same name.
#
# The output naming is completely controllable by the user with the
# dest.path and dest.name parameters. In the default case, these are
# set to:
#
#  dest.path= ./
#  dest.name = %Y-%j-%H-%M-%S-<filename>
#
# There are two macros that can be used in any of the parts and will be
# expanded at run-time:
#
#   <filename>     - The name of the file given in the original message
#   <rule>         - Text part of a rule match (see below).
#
# Matching rules let you distribute the received files into different
# directories based on on the filenames. The 'match' config parameter is a
# list of tuples, each having a fnmatch-style rule and a string to be
# pasted into the 'rule' section of the output name. The default rule is:
#
# match:   [('*','')]
#
# which matches any input file and doesn't insert anything into the pathname.
# Instead, if you wanted to store incoming jpeg's into an image directory,
# and text messages in another, you could give the rules:
#
# match: [('*.jpg','images'),('*.txt','text')]
#
#
# History:
#
#   1.0.0   1999-12-02  TAV
#
#   1.0.1   2000-02-18  TAV
#           Fixed bug in compute_name() that caused the year to only be
#               the first two digits (i.e. 2000 -> 20).
#
#   1.0.2   2000-10-24  TAV
#           Removed the dependance on DateTime module (can do everything
#               with just the standard time module).
#           Changed the naming convention for the output files.
#
#   1.0.3   2000-10-26  TAV
#           Added match rule sets.
#
#   1.0.4   2001-08-10  TAV
#           Fixed NULL -> None.
#           Added transform function.
#
#   1.0.3   2001-08-11  TAV
#           Started to make more robust to failures to copy files. This
#             effort is focused on checking return codes to os.system().
#           Added  abort_on_error config parameter to halt on errors.
#
#   1.0.4   2001-11-09  TAV
#           Changed NewsTool -> NewsTool
#
#   1.0.5   2002-01-25  Todd Valentic
#           Updated to new NewsPollMixin interface.
#
#   1.0.6   2002-08-27  Todd Valentic
#           Configured through the configure script.
#           sri.transport -> Transport
#
#   2016-12-25  Todd Valentic
#               Restored into components package - warning OLD code!
#
#   2022-10-07  Todd Valentic
#               Python3 port
#                   exec -> exec()
#               NewsPoller
#               Note that this old code is currently non-working
#
##############################################################################

import fnmatch
import ftplib
import os
import string
import sys
import time

from datatransport import ProcessClient
from datatransport import NewsPoller
from datatransport import newstool


class FileStore(ProcessClient):
    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poller = NewsPoller(self, callback=self.process)
        self.main = self.news_poller.main

        method = self.get("method", "copy")
        transform = self.get("transform", None)

        self.dest_path = self.get("dest.path", "./")
        self.dest_name = self.get("dest.name", "%Y-%j-%H-%M-%S-<filename>")
        self.abort_on_error = self.get_int("abortOnError", 1)

        if method == "copy":
            self.transfer = self.copy_file
            self.log.debug("Using the copy transfer method")

        elif method == "ftp":
            self.transfer = self.ftp_file
            self.log.debug("Using the ftp transfer method")

            self.dest_server = self.get("dest.server")
            self.dest_username = self.get("dest.username", "anonymous")
            self.dest_password = self.get("dest.password", "transport@")
            self.dest_sleep = self.get_int("dest.sleep", 5)

            if not self.dest_server:
                self.abort("The destination server is not set")

        else:
            self.abort("Unknown transfer method: %s" % method)

        # Test the dest.path and dest.prefix to make sure no badness happens

        curtime = time.localtime(time.time())

        try:
            time.strftime(self.dest_path, curtime)
        except:
            self.abort("There is a problem with the formatting of dest.path")

        try:
            time.strftime(self.dest_name, curtime)
        except:
            self.abort("There is a problem with the formatting of dest.name")

        # Get matching rules

        try:
            self.rules = eval(self.get("match", "[('*','')]"))
        except:
            self.abort("There was a problem reading the match rules")

        # Try to load the transform module

        if transform:
            try:
                module, function = os.path.splitext(transform)
                exec("import %s" % module)
                exec("self.transform = %s" % transform)
            except:
                self.abort("There was a problem loading the transform module.")
        else:
            self.transform = None

    def compute_name(self, header, basename, rule):

        # Assume that the date is at the end and between brackets ([])

        date = string.split(header, "[")[-1]
        dt = time.strptime(date[:-1], "%c")

        filename = os.path.join(self.dest_path, self.dest_name)

        filename = time.strftime(filename, dt)
        filename = string.replace(filename, "<rule>", rule[1])
        filename = string.replace(filename, "<filename>", basename)

        # Do a split in case someone puts path parts in dest_name

        return os.path.split(filename)

    def ftp_file(self, basename, header, rule):

        path, name = self.compute_name(header, basename, rule)

        ftp = ftplib.FTP(self.dest_server)
        ftp.login(self.dest_username, self.dest_password)

        parts = string.split(path, "/")

        if path[0] == "/":
            ftp.cwd("/")

        self.log.debug("Remote path: %s" % path)

        for part in parts[1:]:
            try:
                ftp.cwd(part)
            except:
                ftp.mkd(part)
                ftp.cwd(part)

        self.log.debug("Remote file name: %s" % basename)

        file = open(basename, "r")
        ftp.storbinary("STOR %s" % name, file, 1024)
        file.close()

        os.remove(basename)

        # Some systems don't like a bunch of data ftp'd in a very short
        # amount of time (part of the intrusion detection). So, we sleep
        # for a bit to pace things out.

        time.sleep(self.dest_sleep)

    def copy_file(self, basename, header, rule):

        path, name = self.compute_name(header, basename, rule)

        self.log.debug("basename=%s" % basename)
        self.log.debug("path=%s" % path)
        self.log.debug("name=%s" % name)

        try:
            if not os.path.isdir(path):
                os.path.makedirs(path)
        except:
            msg = "Failed to create path: %s" % path
            if self.abort_on_error:
                self.abort(msg)
            else:
                self.log.error(msg)
                raise newstool.ProcessError

        if os.system("cp %s %s/%s" % (basename, path, name)) != 0:
            msg = "Failed to copy file to: %s/%s. Permission problem?" % (path, name)
            if self.abort_on_error:
                self.abort(msg)
            else:
                self.log.error(msg)
                raise newstool.ProcessError

        os.remove(basename)

        self.log.debug("Copied file to: %s/%s" % (path, name))

    def find_match(self, filename):

        self.log.debug("Trying to match rules to file=%s" % filename)

        for rule in self.rules:
            if fnmatch.fnmatch(filename, rule[0]):
                self.log.debug(" - yes: rule=%s" % rule)
                return rule
            self.log.debug(" - no : rule=%s" % rule)

        self.log.debug("No matching rule found")

        return None

    def process(self, message):
        
        filenames = newstool.save_files(message)

        for filename in filenames:

            rule = self.find_match(filename)

            if rule is None:
                self.log.debug("No matching rule for the file: %s" % filename)
                continue

            if self.transform:
                self.log.debug("Calling transform function")
                self.log.debug("  before: %s" % filename)
                filename = self.transform(filename)
                self.log.debug("  after: %s" % filename)

            try:
                self.transfer(filename, header, rule)
                self.log.info("Transfered file: %s" % header)
            except:
                self.log.error("Failure to transfer file: %s" % header)
                os.system("rm -f %s" % filename)
                raise newstool.ProcessError


def main():
    FileStore(sys.argv).run()



##########################################################################
#
#   FilePost
#
#   This transport component watches for specified files to appear and
#   posts them into a newsgroup. If the newsgroup does not exist, it
#   will be created. Multiple files can be listed and they will be posted
#   as a group of attachments in a single message. This is a useful
#   feature if you have data files that belong together and should stay
#   bundled in the transport network. Once the files are posted, they are
#   removed. An option exists to leave the files as well. For the files
#   to be removed, they need to have write permission for the transport user.
#
#   The component is based on the original FileWatch componment. It
#   expands on it by allowing you to specify multiple groups of files
#   to post, each with its own destination news group. This approach
#   scales much better when there are a number of groupings.
#
#   It uses the standard transport ConfigComponent to specify the
#   groups. Each grouping can list:
#
#       post.newsgroup (and other related options for NewsPoster)
#       remove_files - remove files after posting (default True)
#       group_files - post all files to one message (default False)
#       files - list of file names (including path) to look for. If
#               multiple names are listed, they should be separated
#               by spaces. The name can include normal shell wildcards.
#
#   2010-08-16  Todd Valentic
#               Initial implementation
#
#   2017-01-11  Todd Valentic
#               Handle empty filegroup list
#
#   2022-10-12  Todd Valentic
#               Python3 port
#                   Transport -> datatransport
#                   removeFile -> remove_file
#                   NewsPoster
#
##########################################################################

import os
import sys
import glob
import time

from datatransport import ProcessClient
from datatransport import NewsPoster
from datatransport import ConfigComponent
from datatransport.utilities import remove_file


class Watcher(ConfigComponent):
    def __init__(self, name, parent):
        ConfigComponent.__init__(self, "watch", name, parent)
    
        self.news_poster = NewsPoster(self)

        self.file_patterns = self.get_list("files")
        self.remove_files = self.get_boolean("removefiles", True)
        self.group_files = self.get_boolean("groupfiles", False)

        self.log.info("Watch: %s" % name)
        self.log.info("  Posting to %s" % self.get("post.newsgroup"))
        self.log.info("  Watching for: %s" % self.file_patterns)

    def find_files(self):

        filenames = []

        for pattern in self.file_patterns:
            filenames.extend(glob.glob(pattern))

        filenames = [f for f in filenames if os.path.isfile(f)]
        filenames.sort()

        if self.group_files and filenames:
            filenames = [filenames]

        return filenames

    def check(self):

        files = self.find_files()

        if not files:
            self.log.debug("No files present, sleeping")
            return

        for filegroup in files:

            try:
                starttime = time.time()
                self.news_poster.post(filegroup)
                elapsed = time.time() - starttime
                self.log.info("Files posted (%.2f s): %s" % (elapsed, filegroup))
            except:
                self.log.exception("Problem posting files")
                return

            if self.remove_files:
                try:
                    remove_file(filegroup)
                except:
                    self.log.exception("Problem deleting file: %s" % filegroup)


class FilePost(ProcessClient):
    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.rate = self.get_rate("rate")
        self.watches = self.get_components("watches", Watcher)

    def process(self):

        for watch in self.watches.values():
            watch.check()

    def main(self):

        while self.wait(self.rate):
            self.process()


def main():
    FilePost(sys.argv).run()



from .ArchiveGroups	    import ArchiveGroups
from .DirectoryService  import DirectoryService
from .DiskMonitor	    import DiskMonitor
from .FileWatch         import FileWatch
from .FilePost          import FilePost
from .FileStore         import FileStore
from .GroupControl	    import GroupControl
from .InstrumentStatus	import InstrumentStatus
from .NewsgroupMonitor  import NewsgroupMonitor
from .PlotTool          import PlotTool
from .PostDataFiles	    import PostDataFiles
from .RealTimeFeed	    import RealTimeFeed
from .ResourceMonitor	import ResourceMonitor
from .SyncPoller        import SyncPoller
from .WatchURL		    import WatchURL



##########################################################################
#
#   DiskMonitor
#
#   Monitor disk space using results in ResourceMonitor
#   snapshots. Produce trouble alerts when when running low.
#
#   2004-02-01	Todd Valentic
#               Initial implementation.
#
#   2008-11-13  Todd Valentic
#               Use SafeConfigParser
#
#   2016-12-23  Todd Valentic
#               Use external texttable
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#               Python3 updates
#                   ConfigParser -> sapphire.Parser 
#                   StringIO -> io.StringIO
#                   sizeDesc -> size_desc
#                   NewsPoster, NewsPoller
#
##########################################################################

import io
import fnmatch
import sys

import texttable

from datatransport import ProcessClient
from datatransport import NewsPoster
from datatransport import NewsPoller
from datatransport import newstool
from datatransport import ConfigComponent
from datatransport.utilities import size_desc

import sapphire_config as sapphire


class DiskMonitor(ProcessClient):
    def __init__(self, argv):
        ProcessClient.__init__(self, argv)

        self.news_poster = NewsPoster(self)
        self.news_poller = NewsPoller(self, callback=self.process)
        self.main = self.news_poller.main

        self.load_history()

        self.includes = self.get_list("mounts.include", "*")
        self.excludes = self.get_list("mounts.exclude", "")
        self.maxusedpct = self.get_float("max.percent.used", 95)

    def load_history(self):
        self.history = sapphire.Parser()
        self.history.read("history")

    def save_history(self):
        self.history.write(open("history", "w"))

    def update_history(self, prefix, stats, mount):

        if not self.history.has_section(mount):
            self.history.add_section(mount)

        for option in stats.options(mount):
            value = stats.get(mount, option)
            self.history.set(mount, f"{prefix}.{option}", value)

    def checkmount(self, stats, mount):

        if self.history.has_section(mount):
            prevalarm = self.history.get_int(mount, "last.alarm")
        else:
            prevalarm = 0

        usedpct = stats.get_float(mount, "usedpct")
        curalarm = usedpct >= self.maxusedpct

        setalarm = not prevalarm and curalarm
        clearalarm = prevalarm and not curalarm

        if setalarm:
            self.log.info("Warning: %s is %d%% full" % (mount, usedpct))
            notes = "Alarm set"
        elif clearalarm:
            self.log.info("Cleared: %s is now only %d%% full" % (mount, usedpct))
            notes = "Cleared alarm"
        elif curalarm and prevalarm:
            notes = "Low space alarm"
        else:
            notes = ""

        self.update_history("last", stats, mount)
        self.history.set(mount, "last.alarm", str(curalarm))
        self.history.set(mount, "last.notes", notes)

        return setalarm or clearalarm

    def check(self, stats):

        self.log.debug("Checking disk usage (limit is %d%%)" % self.maxusedpct)

        # Patchup for development debugging
        if not stats.has_section("System"):
            stats.add_section("System")

        mounts = stats.get("System", "mounts").split()

        for include in self.includes:
            mounts = filter(lambda x: fnmatch.fnmatch(x, include), mounts)
        for exclude in self.excludes:
            mounts = filter(lambda x: not fnmatch.fnmatch(x, exclude), mounts)

        report = 0

        for mount in mounts:
            report += self.checkmount(stats, mount)

        if report:
            message = self.alert_message(mounts)
            try:
                self.news_poster.post_text(message, date=self.timestamp)
            except:
                self.log.exception("Problem posting message")

    def alert_message(self, mounts):

        timestr = self.timestamp.strftime("%Y-%m-%d %H:%M")

        message = "\n"
        message += "Disk Space Report - %s\n\n" % timestr

        table = texttable.Texttable()
        table.header(["Mount", "Total", "Avail", "Used", "% Full", "Notes"])
        table.set_cols_align(["l", "r", "r", "r", "r", "l"])

        for mount in mounts:

            total = long(self.history.get(mount, "last.totalbytes"))
            used = long(self.history.get(mount, "last.usedbytes"))
            free = long(self.history.get(mount, "last.freebytes"))
            usedpct = self.history.get_float(mount, "last.usedpct")

            row = [mount]
            row.append(size_desc(total))
            row.append(size_desc(used))
            row.append(size_desc(free))
            row.append("%d%%" % usedpct)
            row.append(self.history.get(mount, "last.notes"))

            table.add_row(row)

        message += table.draw()
        message += "\n"

        return message

    def process(self, message):

        body = message.get_payload()
        file = io.StringIO(body)
        stats = sapphire.Parser()
        stats.readfp(file)

        self.timestamp = newstool.message_date(message)
        self.history.set("DEFAULT", "last.time", self.timestamp)

        self.check(stats)
        self.save_history()


def main():
    DiskMonitor(sys.argv).run()



#!/usr/bin/env python

#########################################################################
#
#   News group gateway service
#
#   This service provides an XML-RPC gateway to news groups.
#   The contents of the latest available message in a group
#   are cached and made available to client programs.
#
#   2007-04-07  Todd Valentic
#               Initial implementation.
#
#   2007-12-06	Todd Valentic
#               Make read() return a JSON representation of the message.
#
#   2009-11-24  Todd Valentic
#               Use currentTime instead of datetime.now()
#
#   2022-10-07  Todd Valentic
#               Reorder imports
#               NewsPoller
#
#########################################################################

import pickle
import shelve
import sys

from threading import Thread, Lock
from datetime import datetime
from datetime import timedelta

from datatransport import ProcessClient
from datatransport import XMLRPCServerMixin
from datatransport import NewsPoller
from datatransport import AccessMixin


def synchronized(lock):
    def wrap(f):
        def new_function(*args, **kw):
            lock.acquire()
            try:
                return f(*args, **kw)
            finally:
                lock.release()

        return new_function

    return wrap


class Data(AccessMixin):

    lock = Lock()

    def __init__(self, parent):
        AccessMixin.__init__(self, parent)
        self.messages = shelve.open("cache")

    @synchronized(lock)
    def write(self, message):
        newsgroup = message["Newsgroups"]
        self.messages[newsgroup] = self.write_handler(newsgroup, message)
        self.messages.sync()

    def write_handler(self, newsgroup, message):
        return message

    @synchronized(lock)
    def clear(self):
        self.messages.clear()
        self.messages.sync()

    @synchronized(lock)
    def read(self, newsgroup):
        return self.messages[newsgroup]

    @synchronized(lock)
    def list(self):
        return self.messages.keys()

    @synchronized(lock)
    def get_string(self, newsgroup):
        return self.messages[newsgroup].as_string()

    @synchronized(lock)
    def unpickle(self, newsgroup):
        return pickle.loads(self.messages[newsgroup].get_payload())


class Server(Thread, AccessMixin):
    def __init__(self, parent, data):
        Thread.__init__(self)
        AccessMixin.__init__(self, parent)

        self.xmlserver = XMLRPCServerMixin(self)
        self.main = self.xmlserver.main

        self.setDaemon(True)

        self.data = data

        self.xmlserver.register_function(self.read, "read")
        self.xmlserver.register_function(self.data.unpickle, "unpickle")
        self.xmlserver.register_function(self.data.list, "list")
        self.xmlserver.register_function(self.data.get_string, "message")

        self.add_handlers()

    def add_handlers(self):
        return

    def read(self, newsgroup):
        message = self.data.read(newsgroup)
        headers = {}
        for key, value in message.items():
            headers[key] = value
        return {"headers": headers, "body": message.get_payload()}


class NewsGateway(ProcessClient):
    def __init__(self, dataFactory, serverFactory, argv):
        ProcessClient.__init__(self, argv)

        self.news_poller = NewsPoller(self, callback=self.process, idle=self.idle)
        self.main = self.news_poller.main

        self.data = dataFactory(self)
        self.timeout = self.get_timedelta("timeout")
        self.in_timeout = False

        if self.timeout:
            self.log.info("Timeout set to %s" % self.timeout)
            self.next_time = self.current_time() + self.timeout
        else:
            self.log.info("No timeout set")

        serverFactory(self, self.data).start()

    def process(self, message):
        self.log.info("Set from %s" % message["Newsgroups"])
        self.data.write(message)

        if self.timeout:
            self.next_time = self.current_time() + self.timeout
            self.in_timeout = False

    def idle(self):

        if self.in_timeout:
            return

        if self.timeout and self.current_time() > self.next_time:
            self.log.info("timeout reached, clearing values")
            self.data.clear()
            self.in_timeout = True


def main():
    NewsGateway(Data, Server, sys.argv).run()



#!/usr/bin/env python3

##########################################################################
#
#   Polling example
#
#   Periodically poll for messages from a news group 
#
#   2022-10-10  Todd Valentic
#               Initial implementation
#
##########################################################################

from datatransport import ProcessClient
from datatransport import NewsPoller

import sys

class Client (ProcessClient):

    def __init__(self, args):
        ProcessClient.__init__(self, args)

        self.poller = NewsPoller(self, callback=self.process)
        self.main = self.poller.main

        self.log.info(f'Poll for messages')

    def process(self, message):

        text = message.get_payload()
        self.log.info(f'Received "{text}"')

if __name__ == '__main__':
    Client(sys.argv).run()

